From 81d75612e0c824cbf23490d39769a3758f09e8a2 Mon Sep 17 00:00:00 2001
From: Tristan Matthews <le.businessman@gmail.com>
Date: Wed, 10 Apr 2013 09:07:34 -0400
Subject: [PATCH 1/1] jack: don't use legacy packet API

---
 modules/audio_output/Modules.am |    2 +-
 modules/audio_output/jack.c     |  208 +++++++++++++++++----
 modules/audio_output/packet.c   |  378 ---------------------------------------
 3 files changed, 170 insertions(+), 418 deletions(-)
 delete mode 100644 modules/audio_output/packet.c

diff --git a/modules/audio_output/Modules.am b/modules/audio_output/Modules.am
index 38e9794..eb0bdc6 100644
--- a/modules/audio_output/Modules.am
+++ b/modules/audio_output/Modules.am
@@ -53,7 +53,7 @@ if HAVE_PULSE
 libvlc_LTLIBRARIES += libpulse_plugin.la
 endif
 
-libjack_plugin_la_SOURCES = jack.c packet.c volume.h
+libjack_plugin_la_SOURCES = jack.c volume.h
 libjack_plugin_la_CFLAGS = $(AM_CFLAGS) $(JACK_CFLAGS)
 libjack_plugin_la_LIBADD = $(AM_LIBADD) $(JACK_LIBS) $(LIBM)
 EXTRA_LTLIBRARIES += libjack_plugin.la
diff --git a/modules/audio_output/jack.c b/modules/audio_output/jack.c
index 22200d5..39e64df 100644
--- a/modules/audio_output/jack.c
+++ b/modules/audio_output/jack.c
@@ -39,6 +39,7 @@
 #include <vlc_aout.h>
 
 #include <jack/jack.h>
+#include <jack/ringbuffer.h>
 
 typedef jack_default_audio_sample_t jack_sample_t;
 
@@ -50,14 +51,17 @@ typedef jack_default_audio_sample_t jack_sample_t;
  *****************************************************************************/
 struct aout_sys_t
 {
-    aout_packet_t   packet;
+    jack_ringbuffer_t *p_jack_ringbuffer;
     jack_client_t  *p_jack_client;
     jack_port_t   **p_jack_ports;
     jack_sample_t **p_jack_buffers;
     unsigned int    i_channels;
+    int             i_rate;
     jack_nframes_t latency;
     float soft_gain;
     bool soft_mute;
+    mtime_t paused; /**< Time when (last) paused */
+    size_t bytes_per_frame;
 };
 
 /*****************************************************************************
@@ -65,6 +69,10 @@ struct aout_sys_t
  *****************************************************************************/
 static int  Open         ( vlc_object_t * );
 static void Close        ( vlc_object_t * );
+static void Play         ( audio_output_t * p_aout, block_t * p_block );
+static void Pause        ( audio_output_t *aout, bool paused, mtime_t date );
+static void Flush        ( audio_output_t *p_aout, bool wait );
+static int  TimeGet      ( audio_output_t *, mtime_t * );
 static int  Process      ( jack_nframes_t i_frames, void *p_arg );
 static int  GraphChange  ( void *p_arg );
 
@@ -109,6 +117,7 @@ static int Start( audio_output_t *p_aout, audio_sample_format_t *restrict fmt )
     int i_error;
 
     p_sys->latency = 0;
+    p_sys->paused = VLC_TS_INVALID;
 
     /* Connect to the JACK server */
     snprintf( psz_name, sizeof(psz_name), "vlc_%d", getpid());
@@ -130,17 +139,16 @@ static int Start( audio_output_t *p_aout, audio_sample_format_t *restrict fmt )
     /* JACK only supports fl32 format */
     fmt->i_format = VLC_CODEC_FL32;
     // TODO add buffer size callback
-    fmt->i_rate = jack_get_sample_rate( p_sys->p_jack_client );
-
-    p_aout->time_get = aout_PacketTimeGet;
-    p_aout->play = aout_PacketPlay;
-    p_aout->pause = NULL;
-    p_aout->flush = aout_PacketFlush;
-    aout_PacketInit( p_aout, &p_sys->packet,
-                     jack_get_buffer_size( p_sys->p_jack_client ), fmt );
+    p_sys->i_rate = fmt->i_rate = jack_get_sample_rate( p_sys->p_jack_client );
+
+    p_aout->play = Play;
+    p_aout->pause = Pause;
+    p_aout->flush = Flush;
+    p_aout->time_get = TimeGet;
     aout_SoftVolumeStart( p_aout );
 
     p_sys->i_channels = aout_FormatNbChannels( fmt );
+    aout_FormatPrepare(fmt);
 
     p_sys->p_jack_ports = malloc( p_sys->i_channels *
                                   sizeof(jack_port_t *) );
@@ -158,6 +166,16 @@ static int Start( audio_output_t *p_aout, audio_sample_format_t *restrict fmt )
         goto error_out;
     }
 
+    p_sys->bytes_per_frame = fmt->i_bytes_per_frame;
+    const size_t buf_sz = (AOUT_MAX_ADVANCE_TIME / CLOCK_FREQ * fmt->i_rate * fmt->i_bytes_per_frame);
+    p_sys->p_jack_ringbuffer = jack_ringbuffer_create( buf_sz );
+
+    if( p_sys->p_jack_ringbuffer == NULL )
+    {
+        status = VLC_ENOMEM;
+        goto error_out;
+    }
+
     /* Create the output ports */
     for( i = 0; i < p_sys->i_channels; i++ )
     {
@@ -231,39 +249,151 @@ error_out:
         {
             jack_deactivate( p_sys->p_jack_client );
             jack_client_close( p_sys->p_jack_client );
-            aout_PacketDestroy( p_aout );
         }
+        if( p_sys->p_jack_ringbuffer)
+            jack_ringbuffer_free( p_sys->p_jack_ringbuffer );
+
         free( p_sys->p_jack_ports );
         free( p_sys->p_jack_buffers );
     }
     return status;
 }
 
+static void Play (audio_output_t * p_aout, block_t * p_block)
+{
+    struct aout_sys_t *p_sys = p_aout->sys;
+
+    while (p_block->i_buffer > 0) {
+
+        /* move data to buffer */
+        const size_t bytes = __MIN( jack_ringbuffer_write_space(p_sys->p_jack_ringbuffer), p_block->i_buffer );
+        if (unlikely(jack_ringbuffer_write( p_sys->p_jack_ringbuffer, (const char *) p_block->p_buffer, bytes ) < bytes)) {
+            msg_Warn(p_aout, "Some audio was dropped");
+            break;
+        }
+
+        p_block->i_nb_samples -= bytes / sizeof(jack_sample_t);
+        p_block->p_buffer += bytes;
+        p_block->i_buffer = p_block->i_buffer < bytes ? 0 : p_block->i_buffer - bytes;
+    }
+
+    block_Release(p_block);
+}
+
+/**
+ * Pause or unpause playback
+ */
+static void Pause(audio_output_t *aout, bool paused, mtime_t date)
+{
+    aout_sys_t *sys = aout->sys;
+
+    if (paused) {
+        sys->paused = date;
+    } else {
+        date -= sys->paused;
+        msg_Dbg(aout, "resuming after %"PRId64" us", date);
+        sys->paused = VLC_TS_INVALID;
+    }
+}
+
+static void Flush(audio_output_t *p_aout, bool wait)
+{
+    struct aout_sys_t * p_sys = p_aout->sys;
+    jack_ringbuffer_t *rb = p_sys->p_jack_ringbuffer;
+    int i_error;
+
+    /* Sleep until the ring buffer has been emptied */
+    if( wait )
+    {
+        const mtime_t wait_time = jack_frames_to_time(p_sys->p_jack_client, jack_get_buffer_size(p_sys->p_jack_client));
+        while( jack_ringbuffer_write_space( rb ) != rb->size )
+            msleep(wait_time);
+    }
+
+    /* FIXME: maybe the ringbuffer_reset should just be protected by signalling with atomic variables? */
+
+    /* Save connections */
+    const char ***connections_list = malloc( p_sys->i_channels * sizeof(const char **) );
+    if( connections_list == NULL )
+    {
+        msg_Err( p_aout, "Out of memory" );
+        return;
+    }
+
+    for(size_t i = 0; i < p_sys->i_channels; i++ )
+    {
+        const char **port_connections = jack_port_get_all_connections( p_sys->p_jack_client, p_sys->p_jack_ports[i] );
+        connections_list[i] = port_connections;
+    }
+
+    /* Suspend audio processing */
+    i_error = jack_deactivate( p_sys->p_jack_client );
+    if( i_error )
+    {
+        msg_Err( p_aout, "failed to deactivate JACK client (error %d)", i_error );
+        return;
+    }
+
+    /* reset ringbuffer read and write pointers */
+    jack_ringbuffer_reset(rb);
+
+    /* Resume audio processing */
+    i_error = jack_activate( p_sys->p_jack_client );
+    if( i_error )
+    {
+        msg_Err( p_aout, "failed to activate JACK client (error %d)", i_error );
+        return;
+    }
+
+    /* Restore connections */
+    for(size_t i = 0; i < p_sys->i_channels; i++ )
+    {
+        const char **port_connections = connections_list[i];
+        const char* psz_out = jack_port_name( p_sys->p_jack_ports[i] );
+        for (const char **psz_in = port_connections; psz_in && *psz_in; psz_in++)
+        {
+            i_error = jack_connect( p_sys->p_jack_client, psz_out, *psz_in );
+            if( i_error )
+            {
+                msg_Err( p_aout, "failed to connect port %s to port %s (error %d)",
+                        psz_out, *psz_in, i_error );
+            }
+        }
+
+        if (port_connections)
+            jack_free(port_connections);
+    }
+
+    free(connections_list);
+}
+
+static int TimeGet(audio_output_t *p_aout, mtime_t *delay)
+{
+    struct aout_sys_t * p_sys = p_aout->sys;
+    jack_ringbuffer_t *rb = p_sys->p_jack_ringbuffer;
+    if( p_sys->bytes_per_frame == 0 )
+        return -1;
+
+    *delay = (p_sys->latency + (jack_ringbuffer_read_space(rb) / p_sys->bytes_per_frame)) * CLOCK_FREQ / p_sys->i_rate;
+
+    return 0;
+}
 
 /*****************************************************************************
  * Process: callback for JACK
  *****************************************************************************/
 int Process( jack_nframes_t i_frames, void *p_arg )
 {
-    unsigned int i, j, i_nb_samples = 0;
+    unsigned int i, j, frames_from_rb = 0;
+    size_t bytes_read = 0;
+    size_t frames_read;
     audio_output_t *p_aout = (audio_output_t*) p_arg;
     struct aout_sys_t *p_sys = p_aout->sys;
-    jack_sample_t *p_src = NULL;
 
-    jack_nframes_t dframes = p_sys->latency
-                             - jack_frames_since_cycle_start( p_sys->p_jack_client );
+    /* Get the next audio data buffer unless paused */
 
-    jack_time_t dtime = dframes * 1000 * 1000 / jack_get_sample_rate( p_sys->p_jack_client );
-    mtime_t play_date = mdate() + (mtime_t) ( dtime );
-
-    /* Get the next audio data buffer */
-    block_t *p_buffer = aout_PacketNext( p_aout, play_date );
-
-    if( p_buffer != NULL )
-    {
-        p_src = (jack_sample_t *)p_buffer->p_buffer;
-        i_nb_samples = p_buffer->i_nb_samples;
-    }
+    if( p_sys->paused == VLC_TS_INVALID)
+        frames_from_rb = i_frames;
 
     /* Get the JACK buffers to write to */
     for( i = 0; i < p_sys->i_channels; i++ )
@@ -273,28 +403,27 @@ int Process( jack_nframes_t i_frames, void *p_arg )
     }
 
     /* Copy in the audio data */
-    for( j = 0; j < i_nb_samples; j++ )
+    for( j = 0; j < frames_from_rb; j++ )
     {
         for( i = 0; i < p_sys->i_channels; i++ )
         {
-            jack_sample_t *p_dst = p_sys->p_jack_buffers[i];
-            p_dst[j] = *p_src;
-            p_src++;
+            jack_sample_t *p_dst = p_sys->p_jack_buffers[i] + j;
+            bytes_read += jack_ringbuffer_read( p_sys->p_jack_ringbuffer,
+                    (char *) p_dst, sizeof(jack_sample_t) );
         }
     }
 
     /* Fill any remaining buffer with silence */
-    if( i_nb_samples < i_frames )
+    frames_read = (bytes_read / sizeof(jack_sample_t)) / p_sys->i_channels;
+    if( frames_read < i_frames )
     {
         for( i = 0; i < p_sys->i_channels; i++ )
         {
-            memset( p_sys->p_jack_buffers[i] + i_nb_samples, 0,
-                    sizeof( jack_sample_t ) * (i_frames - i_nb_samples) );
+            memset( p_sys->p_jack_buffers[i] + frames_read, 0,
+                    sizeof( jack_sample_t ) * (i_frames - frames_read) );
         }
     }
 
-    if( p_buffer )
-        block_Release( p_buffer );
     return 0;
 }
 
@@ -308,15 +437,15 @@ static int GraphChange( void *p_arg )
   audio_output_t *p_aout = (audio_output_t*) p_arg;
   struct aout_sys_t *p_sys = p_aout->sys;
   unsigned int i;
-  jack_nframes_t port_latency;
+  jack_latency_range_t port_latency;
 
   p_sys->latency = 0;
 
   for( i = 0; i < p_sys->i_channels; ++i )
   {
-    port_latency = jack_port_get_total_latency( p_sys->p_jack_client,
-                                                  p_sys->p_jack_ports[i] );
-    p_sys->latency = __MAX( p_sys->latency, port_latency );
+    jack_port_get_latency_range( p_sys->p_jack_ports[i], JackPlaybackLatency,
+                                 &port_latency );
+    p_sys->latency = __MAX( p_sys->latency, port_latency.max );
   }
 
   msg_Dbg(p_aout, "JACK graph reordered. Our maximum latency=%d.", p_sys->latency);
@@ -345,7 +474,8 @@ static void Stop( audio_output_t *p_aout )
     }
     free( p_sys->p_jack_ports );
     free( p_sys->p_jack_buffers );
-    aout_PacketDestroy( p_aout );
+    if( p_sys->p_jack_ringbuffer )
+        jack_ringbuffer_free( p_sys->p_jack_ringbuffer );
 }
 
 static int Open(vlc_object_t *obj)
diff --git a/modules/audio_output/packet.c b/modules/audio_output/packet.c
deleted file mode 100644
index 48f7491..0000000
--- a/modules/audio_output/packet.c
+++ /dev/null
@@ -1,378 +0,0 @@
-/*****************************************************************************
- * packet.c : helper for legacy audio output plugins
- *****************************************************************************
- * Copyright (C) 2002-2012 VLC authors and VideoLAN
- *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
- * GNU Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this program; if not, write to the Free Software Foundation,
- * Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
- *****************************************************************************/
-
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
-#include <limits.h>
-#include <assert.h>
-#include <vlc_common.h>
-#include <vlc_aout.h>
-
-/**
- * Initializes the members of a FIFO.
- */
-static void aout_FifoInit( aout_fifo_t *p_fifo, uint32_t i_rate )
-{
-    assert( i_rate != 0);
-    p_fifo->p_first = NULL;
-    p_fifo->pp_last = &p_fifo->p_first;
-    date_Init( &p_fifo->end_date, i_rate, 1 );
-    date_Set( &p_fifo->end_date, VLC_TS_INVALID );
-}
-
-/**
- * Pushes a packet into the FIFO.
- */
-static void aout_FifoPush( aout_fifo_t * p_fifo, block_t * p_buffer )
-{
-    *p_fifo->pp_last = p_buffer;
-    p_fifo->pp_last = &p_buffer->p_next;
-    *p_fifo->pp_last = NULL;
-    /* Enforce the continuity of the stream. */
-    if( date_Get( &p_fifo->end_date ) != VLC_TS_INVALID )
-    {
-        p_buffer->i_pts = date_Get( &p_fifo->end_date );
-        p_buffer->i_length = date_Increment( &p_fifo->end_date,
-                                             p_buffer->i_nb_samples );
-        p_buffer->i_length -= p_buffer->i_pts;
-    }
-    else
-    {
-        date_Set( &p_fifo->end_date, p_buffer->i_pts + p_buffer->i_length );
-    }
-}
-
-/**
- * Trashes all buffers.
- */
-static void aout_FifoReset( aout_fifo_t * p_fifo )
-{
-    block_t * p_buffer;
-
-    date_Set( &p_fifo->end_date, VLC_TS_INVALID );
-    p_buffer = p_fifo->p_first;
-    while ( p_buffer != NULL )
-    {
-        block_t * p_next = p_buffer->p_next;
-        block_Release( p_buffer );
-        p_buffer = p_next;
-    }
-    p_fifo->p_first = NULL;
-    p_fifo->pp_last = &p_fifo->p_first;
-}
-
-/**
- * Move forwards or backwards all dates in the FIFO.
- */
-static void aout_FifoMoveDates( aout_fifo_t *fifo, mtime_t difference )
-{
-    if( date_Get( &fifo->end_date ) == VLC_TS_INVALID )
-    {
-        assert( fifo->p_first == NULL );
-        return;
-    }
-
-    date_Move( &fifo->end_date, difference );
-    for( block_t *block = fifo->p_first; block != NULL; block = block->p_next )
-        block->i_pts += difference;
-}
-
-/**
- * Gets the next buffer out of the FIFO
- */
-static block_t *aout_FifoPop( aout_fifo_t * p_fifo )
-{
-    block_t *p_buffer = p_fifo->p_first;
-    if( p_buffer != NULL )
-    {
-        p_fifo->p_first = p_buffer->p_next;
-        if( p_fifo->p_first == NULL )
-            p_fifo->pp_last = &p_fifo->p_first;
-    }
-    return p_buffer;
-}
-
-/**
- * Destroys a FIFO and its buffers
- */
-static void aout_FifoDestroy( aout_fifo_t * p_fifo )
-{
-    block_t * p_buffer;
-
-    p_buffer = p_fifo->p_first;
-    while ( p_buffer != NULL )
-    {
-        block_t * p_next = p_buffer->p_next;
-        block_Release( p_buffer );
-        p_buffer = p_next;
-    }
-
-    p_fifo->p_first = NULL;
-    p_fifo->pp_last = &p_fifo->p_first;
-}
-
-static inline aout_packet_t *aout_packet (audio_output_t *aout)
-{
-    return (aout_packet_t *)(aout->sys);
-}
-
-void aout_PacketInit (audio_output_t *aout, aout_packet_t *p, unsigned samples,
-                      const audio_sample_format_t *fmt)
-{
-    assert (p == aout_packet (aout));
-
-    vlc_mutex_init (&p->lock);
-    p->format = *fmt;
-    aout_FormatPrepare (&p->format);
-    aout_FifoInit (&p->partial, p->format.i_rate);
-    aout_FifoInit (&p->fifo, p->format.i_rate);
-    p->pause_date = VLC_TS_INVALID;
-    p->samples = samples;
-    p->starving = true;
-}
-
-void aout_PacketDestroy (audio_output_t *aout)
-{
-    aout_packet_t *p = aout_packet (aout);
-
-    aout_FifoDestroy (&p->partial);
-    aout_FifoDestroy (&p->fifo);
-    vlc_mutex_destroy (&p->lock);
-}
-
-int aout_PacketTimeGet (audio_output_t *aout, mtime_t *restrict delay)
-{
-    aout_packet_t *p = aout_packet (aout);
-    mtime_t time_report;
-
-    /* Problem: This measurement is imprecise and prone to jitter.
-     * Solution: Do not use aout_Packet...(). */
-    vlc_mutex_lock (&p->lock);
-    time_report = date_Get (&p->fifo.end_date);
-    vlc_mutex_unlock (&p->lock);
-
-    if (time_report == VLC_TS_INVALID)
-        return -1;
-    *delay = time_report - mdate ();
-    return 0;
-}
-
-static block_t *aout_OutputSlice (audio_output_t *);
-
-void aout_PacketPlay (audio_output_t *aout, block_t *block)
-{
-    aout_packet_t *p = aout_packet (aout);
-
-    vlc_mutex_lock (&p->lock);
-    aout_FifoPush (&p->partial, block);
-    while ((block = aout_OutputSlice (aout)) != NULL)
-        aout_FifoPush (&p->fifo, block);
-    vlc_mutex_unlock (&p->lock);
-}
-
-void aout_PacketFlush (audio_output_t *aout, bool drain)
-{
-    aout_packet_t *p = aout_packet (aout);
-
-    vlc_mutex_lock (&p->lock);
-    if (drain)
-    {
-        mtime_t pts = date_Get (&p->fifo.end_date);
-        vlc_mutex_unlock (&p->lock);
-        if (pts != VLC_TS_INVALID)
-            mwait (pts);
-    }
-    else
-    {
-        aout_FifoReset (&p->partial);
-        aout_FifoReset (&p->fifo);
-        vlc_mutex_unlock (&p->lock);
-    }
-}
-
-
-/**
- * Rearranges audio blocks in correct number of samples.
- * @note (FIXME) This is left here for historical reasons. It belongs in the
- * output code. Besides, this operation should be avoided if possible.
- */
-static block_t *aout_OutputSlice (audio_output_t *p_aout)
-{
-    aout_packet_t *p = aout_packet (p_aout);
-    aout_fifo_t *p_fifo = &p->partial;
-    const unsigned samples = p->samples;
-    assert( samples > 0 );
-
-    /* Retrieve the date of the next buffer. */
-    date_t exact_start_date = p->fifo.end_date;
-    mtime_t start_date = date_Get( &exact_start_date );
-
-    /* Check if there is enough data to slice a new buffer. */
-    block_t *p_buffer = p_fifo->p_first;
-    if( p_buffer == NULL )
-        return NULL;
-
-    /* Find the earliest start date available. */
-    if ( start_date == VLC_TS_INVALID )
-    {
-        start_date = p_buffer->i_pts;
-        date_Set( &exact_start_date, start_date );
-    }
-    /* Compute the end date for the new buffer. */
-    mtime_t end_date = date_Increment( &exact_start_date, samples );
-
-    /* Check that we have enough samples (TODO merge with next loop). */
-    for( unsigned available = 0; available < samples; )
-    {
-        p_buffer = p_buffer->p_next;
-        if( p_buffer == NULL )
-            return NULL;
-
-        available += p_buffer->i_nb_samples;
-    }
-
-    if( AOUT_FMT_LINEAR( &p->format ) )
-    {
-        const unsigned framesize = p->format.i_bytes_per_frame;
-        /* Build packet with adequate number of samples */
-        unsigned needed = samples * framesize;
-
-        p_buffer = block_Alloc( needed );
-        if( unlikely(p_buffer == NULL) )
-            /* XXX: should free input buffers */
-            return NULL;
-        p_buffer->i_nb_samples = samples;
-
-        for( uint8_t *p_out = p_buffer->p_buffer; needed > 0; )
-        {
-            block_t *p_inbuf = p_fifo->p_first;
-            if( unlikely(p_inbuf == NULL) )
-            {
-                msg_Err( p_aout, "packetization error" );
-                memset( p_out, 0, needed );
-                break;
-            }
-
-            const uint8_t *p_in = p_inbuf->p_buffer;
-            size_t avail = p_inbuf->i_nb_samples * framesize;
-            if( avail > needed )
-            {
-                memcpy( p_out, p_in, needed );
-                p_fifo->p_first->p_buffer += needed;
-                p_fifo->p_first->i_buffer -= needed;
-                needed /= framesize;
-                p_fifo->p_first->i_nb_samples -= needed;
-
-                mtime_t t = needed * CLOCK_FREQ / p->format.i_rate;
-                p_fifo->p_first->i_pts += t;
-                p_fifo->p_first->i_length -= t;
-                break;
-            }
-
-            memcpy( p_out, p_in, avail );
-            needed -= avail;
-            p_out += avail;
-            /* Next buffer */
-            block_Release( aout_FifoPop( p_fifo ) );
-        }
-    }
-    else
-        p_buffer = aout_FifoPop( p_fifo );
-
-    p_buffer->i_pts = start_date;
-    p_buffer->i_length = end_date - start_date;
-
-    return p_buffer;
-}
-
-/**
- * Dequeues the next audio packet (a.k.a. audio fragment).
- * The audio output plugin must first call aout_PacketPlay() to queue the
- * decoded audio samples. Typically, audio_output_t.play is set to, or calls
- * aout_PacketPlay().
- * @note This function is considered legacy. Please do not use this function in
- * new audio output plugins.
- * @param p_aout audio output instance
- * @param start_date expected PTS of the audio packet
- */
-block_t *aout_PacketNext (audio_output_t *p_aout, mtime_t start_date)
-{
-    aout_packet_t *p = aout_packet (p_aout);
-    aout_fifo_t *p_fifo = &p->fifo;
-    block_t *p_buffer;
-    const bool b_can_sleek = !AOUT_FMT_LINEAR(&p->format);
-    const mtime_t now = mdate ();
-    const mtime_t threshold =
-        (b_can_sleek ? start_date : now) - AOUT_MAX_PTS_DELAY;
-
-    vlc_mutex_lock( &p->lock );
-    if( p->pause_date != VLC_TS_INVALID )
-        goto out; /* paused: do not dequeue buffers */
-
-    for (;;)
-    {
-        p_buffer = p_fifo->p_first;
-        if (p_buffer == NULL)
-            goto out; /* nothing to play */
-
-        if (p_buffer->i_pts >= threshold)
-            break;
-
-        /* Drop the audio sample if the audio output is really late.
-         * In the case of b_can_sleek, we don't use a resampler so we need to
-         * be a lot more severe. */
-        msg_Dbg (p_aout, "audio output is too slow (%"PRId64" us): "
-                 " trashing %"PRId64" us", threshold - p_buffer->i_pts,
-                 p_buffer->i_length);
-        block_Release (aout_FifoPop (p_fifo));
-    }
-
-    mtime_t delta = start_date - p_buffer->i_pts;
-    /* This assumes that all buffers have the same duration. This is true
-     * since aout_PacketPlay() (aout_OutputSlice()) is used. */
-    if (0 >= delta + p_buffer->i_length)
-    {
-        if (!p->starving)
-        {
-            msg_Dbg (p_aout, "audio output is starving (%"PRId64"), "
-                     "playing silence", delta);
-            p->starving = true;
-        }
-        goto out; /* nothing to play _yet_ */
-    }
-
-    p->starving = false;
-    p_buffer = aout_FifoPop( p_fifo );
-
-    if (!b_can_sleek
-     && (delta < -AOUT_MAX_PTS_ADVANCE || AOUT_MAX_PTS_DELAY < delta))
-    {
-        msg_Warn (p_aout, "audio output out of sync, "
-                          "adjusting dates (%"PRId64" us)", delta);
-        aout_FifoMoveDates (&p->partial, delta);
-        aout_FifoMoveDates (p_fifo, delta);
-    }
-    vlc_mutex_unlock( &p->lock );
-    return p_buffer;
-out:
-    vlc_mutex_unlock( &p->lock );
-    return NULL;
-}
-- 
1.7.10.4

